import abc
from typing import Generic, TypeVar
import json
from json import JSONEncoder
import jsonpickle

T = TypeVar('T')

class ISerializer(Generic[T]):
    @abc.abstractmethod
    def serialize(self, data: T) -> str:
        pass
    
    @abc.abstractmethod
    def deserialize(self, data: str) -> T:
        pass
    
class BlobJsonEncoder(JSONEncoder):
    def default(self, o):
        return o.__dict__
    
class JsonSerializer(ISerializer[T]):
    def serialize(self, data: T) -> str:
        return BlobJsonEncoder().encode(data)
    
    def deserialize(self, data: str) -> T:
        return json.loads(data)

class JsonPickleSerializer(ISerializer[T]):
    def serialize(self, data: T) -> str:
        return jsonpickle.encode(data)
    
    def deserialize(self, data: str) -> T:
        return jsonpickle.decode(data)


from dataclasses import dataclass
from bloboluni import BlobStorage, JsonBlobStorage, JsonPickleBlobStorage
import os, uuid
from dotenv import load_dotenv
load_dotenv()

@dataclass
class MyProfession:
    name: str
    description: str

@dataclass
class MyClass:
    name: str
    age: int
    profession: MyProfession
    
class SomeThingElse:
    def __init__(self, isCool, name) -> None:
        self.isCool = isCool
        self.name = name
        
    def amethod(self):
        return "A method"

if __name__ == "__main__":
    data = MyClass("blob", 30, MyProfession("Developer", "A developer"))

    connectionstring = os.environ["AZURE_STORAGE_CONNECTION_STRING"]
    print(connectionstring)
    storage = BlobStorage(connectionstring, "mycontainer")
    
    key = uuid.uuid4().hex
    blob = storage.get(key)
    assert blob is None
    storage.upsert(key, data)
    blob = storage.get(key)
    assert blob is not None
    print(type(blob))
    storage.delete(key)
    blob = storage.get(key)
    assert blob is None
    
    storage.upsert("something", SomeThingElse(True, "something"))
    
    somethingelse = storage.get("something")
    print(type(somethingelse))
    print(somethingelse)
    
    anotherthing = {"doDrugs": "cool"}
    storage.upsert("anotherthing", anotherthing)
    
    anotherthing = storage.get("anotherthing")
    print(type(anotherthing))
    print(anotherthing)
    
    
    